package net.zenrindatacom.locationservice.sample.activity;

import android.Manifest.permission;
import androidx.annotation.NonNull;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.PermissionChecker;

import net.zenrindatacom.locationservice.sample.application.ILocationNotifyListener;
import net.zenrindatacom.locationservice.sample.history.HistoryManager.IHistoryNotifyListener;
import net.zenrindatacom.locationservice.sample.preference.AppPreference;
import net.zenrindatacom.locationservice.sample.preference.LocationLibPreference;
import net.zenrindatacom.locationservice.sample.locationlib.LocationLibIntentAnalyzer;
import net.zenrindatacom.locationservice.sample.locationlib.LocationLibConst;
import net.zenrindatacom.locationservice.sample.application.SampleApplication;
import net.zenrindatacom.locationservice.sample.R;
import net.zenrindatacom.locationservice.sample.locationlib.LocationLibSetting;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

/**
 * メイン画面
 *
 * @author K.Hatano
 */
public class MainActivity extends AppCompatActivity {
    /** パーミッションチェック：測位開始 */
    private static final int PERMISSIONS_REQUEST_START_LOCATION = 1;
    /** パーミッションチェック：現在地測位 */
    private static final int PERMISSIONS_REQUEST_GET_LOCATION = 2;
    /** 結果表示部 */
    private ListView mResultView;
    /** 測位開始時のIntentインスタンス */
    private Intent mStartLocationPositioningIntent;
    /** 結果表示用アダプタ */
    private static ResultListAdapter mResultListAdapter;
    /** 結果リスト */
    static List<String> mResultList = new ArrayList<>();

    /**
     * 位置測位の結果通知を受け取るリスナー
     */
    private final IHistoryNotifyListener mHistoryNotifyListener = new IHistoryNotifyListener() {

        final Handler mHandler = new Handler();

        @Override
        public void onTokenServerError() {
            mHandler.post(() ->
                    addResult(R.string.main_label_geolocation_send_result, "トークン取得通信に失敗しました"));
        }

        @Override
        public void onLocationServerError() {
            mHandler.post(() ->
                    addResult(R.string.main_label_geolocation_send_result, "位置履歴サーバ送信に失敗しました"));
        }

        @Override
        public void onSuccess() {
            mHandler.post(() ->
                    addResult(R.string.main_label_geolocation_send_result, "位置履歴サーバ送信に成功しました"));
        }
    };

    /**
     * Activityの生成
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main_activity);
        mResultView = findViewById(R.id.resultListView);

        // 結果リストのアダプタセット
        setResultListAdapter();

        SampleApplication application = (SampleApplication) getApplication();
        //位置測位結果の通知を受け取るリスナーを設定
        application.setLocationNotifyListener(mLocationNotifyListener);
        //履歴送信結果の通知を受け取るリスナーを設定
        application.setHistoryNotifyListener(mHistoryNotifyListener);

        //「ライブラリ設定」プリファレンス初期化（初回起動時のみ初期化される）
        LocationLibPreference.getInstance().init();

        //「アプリ設定」プリファレンス初期化（初回起動時のみ初期化される）
        AppPreference.getInstance().init();

        //各ボタンのonClickを定義
        //測位開始ボタン
        findViewById(R.id.positioningStartButton).setOnClickListener(this::pushPositioningStart);
        //現在地取得（単測位）ボタン
        findViewById(R.id.getLocationButton).setOnClickListener(this::pushGetLocation);
        //測位停止ボタン
        findViewById(R.id.positioningStopButton).setOnClickListener(this::pushPositioningStop);
        //バージョン取得ボタン
        findViewById(R.id.getVersionButton).setOnClickListener(this::pushGetVersion);
        //設定値取得ボタン
        findViewById(R.id.getLibrarySettingsButton).setOnClickListener(this::pushGetLibrarySettings);
    }

    /**
     * Activityの破棄
     */
    @Override
    protected void onDestroy() {
        SampleApplication application = (SampleApplication) getApplication();
        //位置測位結果の通知を受け取るリスナーを解除
        application.setLocationNotifyListener(null);
        //履歴送信結果の通知を受け取るリスナーを解除
        application.setHistoryNotifyListener(null);
        super.onDestroy();
    }

    /**
     * オプションメニューの生成
     * @param menu メニュー
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    /**
     * 「測位開始」ボタンがタップされた
     *
     * @param view タップされたボタン
     */
    public void pushPositioningStart(View view) {

        //結果表示の初期化
        clearListView();

        // ・メソッドの呼び元が、ボタンタップ（viewがnullでない）の時
        // 　→　Permissionチェックを行う
        // ・メソッドの呼び元が、Permissionチェック後の再帰呼び出し（viewがnull）の時
        // 　→　Permissionチェックを再度行わない
        if(view != null) {
            if (!checkSelfPermission(this, PERMISSIONS_REQUEST_START_LOCATION)) {
                // 不許可の場合は処理終了
                return;
            }
        }

        Intent intent = new Intent();

        //測位開始のIntentパラメータを設定する
        LocationLibSetting.setIntentStartValue(intent);

        //Android8以降はstartForegroundServiceでサービス起動
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            startForegroundService(intent);
        } else {
            startService(intent);
        }
        mStartLocationPositioningIntent = intent;

        //Intentに設定した情報を画面に表示する
        String settingString = LocationLibIntentAnalyzer.getLocationStartSetting(intent, intent);
        addResult(R.string.main_label_startup_parameter, settingString);
    }

    /**
     * 「現在地取得」ボタンがタップされた
     *
     * @param view タップされたボタン
     */
    public void pushGetLocation(View view) {

        // ・メソッドの呼び元が、ボタンタップ（viewがnullでない）の時
        // 　→　Permissionチェックを行う
        // ・メソッドの呼び元が、Permissionチェック後の再帰呼び出し（viewがnull）の時
        // 　→　Permissionチェックを再度行わない
        if(view != null) {
            if (!checkSelfPermission(this, PERMISSIONS_REQUEST_GET_LOCATION)) {
                // 不許可の場合は処理終了
                return;
            }
        }

        Intent intent = new Intent();

        //現在地取得のIntentパラメータを設定する
        LocationLibSetting.setIntentGetLocationValue(intent);

        //サービス起動
        startService(intent);
    }

    /**
     * 「測位停止」ボタンがタップされた
     *
     * @param view タップされたボタン
     */
    public void pushPositioningStop(@SuppressWarnings("unused") View view) {
        Intent intent = new Intent();

        //測位停止のIntentパラメータを設定する
        LocationLibSetting.setIntentStopValue(intent);

        //サービス起動
        startService(intent);

        mStartLocationPositioningIntent = null;

        //起動停止
        addResult(R.string.main_label_geolocation_stop, "");
    }

    /**
     * 「バージョン取得」ボタンがタップされた
     *
     * @param view タップされたボタン
     */
    public void pushGetVersion(@SuppressWarnings("unused") View view) {
        Intent intent = new Intent();

        //バージョン取得のIntentパラメータを設定する
        LocationLibSetting.setIntentGetVersionValue(intent);

        //サービス起動
        startService(intent);
    }

    /**
     * 「設定値取得」ボタンがタップされた
     *
     * @param view タップされたボタン
     */
    public void pushGetLibrarySettings(@SuppressWarnings("unused") View view) {
        //「設定値」と「動作値」のどちらを取得するか確認する
        final CharSequence[] items = {this.getString(R.string.dlg_list_get_set_value), this.getString(R.string.dlg_list_get_operation_value)};
        AlertDialog.Builder listDlg = new AlertDialog.Builder(this);
        listDlg.setItems(
                items,
                (dialog, which) -> {
                    Intent intent = new Intent();
                    //Intentにパラメータを設定する（which＝0なら設定値、1なら動作値）
                    LocationLibSetting.setIntentGetLibrarySettingsValue(intent, which==1);
                    //サービス起動
                    startService(intent);
                });
        listDlg.create().show();
    }

    /**
     * メニュー内のボタンがタップされた
     * @param item タップされたメニューアイテム
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int itemId = item.getItemId();
        if (itemId == R.id.show_location_lib_setting) {
            //ライブラリ設定画面を表示
            Intent intent = new Intent(this, LocationLibSettingActivity.class);
            startActivity(intent);
        } else if (itemId == R.id.show_app_setting) {
            //アプリ設定画面を表示
            Intent intent = new Intent(this, AppSettingActivity.class);
            startActivity(intent);
        }
        return super.onOptionsItemSelected(item);
    }

    /**
     * パーミッションの確認
     */
    @SuppressWarnings("BooleanMethodIsAlwaysInverted")
    private boolean checkSelfPermission(Context context, int requestCode) {

        // 権限許可状況チェック
        ArrayList<String> permissions = new ArrayList<>();
        // 位置測位許可
        if (PermissionChecker.PERMISSION_GRANTED
                != PermissionChecker.checkSelfPermission(context, permission.ACCESS_FINE_LOCATION)) {
            permissions.add(permission.ACCESS_FINE_LOCATION);
        }

        // 通知許可（フォアグラウンドサービスの通知ドロワーへの表示）
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            if (requestCode == PERMISSIONS_REQUEST_START_LOCATION) {
                //連続測位の場合
                if (PermissionChecker.PERMISSION_GRANTED
                        != PermissionChecker.checkSelfPermission(context, permission.POST_NOTIFICATIONS)) {
                    permissions.add(permission.POST_NOTIFICATIONS);
                }
            }
        }

        // 不許可のPermissionが存在する場合はシステムへ許可をリクエストする
        if (permissions.size() > 0) {
            requestPermissions(permissions.toArray(new String[0]), requestCode);
            return false;
        }
        return true;
    }

    /**
     * パーミッションリクエストの結果
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {

        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (PERMISSIONS_REQUEST_START_LOCATION != requestCode
                && PERMISSIONS_REQUEST_GET_LOCATION != requestCode) {
            return;
        }

        /*
         * grantResultsにユーザーが選択した結果が返ってくる。
         * ただし、アプリが互換モードで動作している場合は必ず 「0:許可されている」が返却される。
         * そのため、grantResults を判定には使用せず、support-v4ライブラリのメソッドで判定を行っている。
         *
         * ※互換モードでなければgrantResultsは正常に値を返すため使用することはできる。
         */
        // 位置測位許可
        if (PermissionChecker.PERMISSION_GRANTED
                != PermissionChecker.checkSelfPermission(this, permission.ACCESS_FINE_LOCATION)) {
            Toast.makeText(getApplication(), R.string.toast_setting_error_permission, Toast.LENGTH_SHORT).show();
            return;
        }

        // 通知許可（フォアグラウンドサービスの通知ドロワーへの表示）
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            if (requestCode == PERMISSIONS_REQUEST_START_LOCATION) {
                //連続測位の場合
                if (PermissionChecker.PERMISSION_GRANTED
                        != PermissionChecker.checkSelfPermission(this, permission.POST_NOTIFICATIONS)) {
                    //通知許可されなくても動作するため、エラーにしないで処理続行
                    Toast.makeText(getApplication(), R.string.toast_setting_info_notification_permission, Toast.LENGTH_SHORT).show();
                }
            }
        }

        // ここまできたら権限が許可されている
        if (requestCode == PERMISSIONS_REQUEST_START_LOCATION) {
            // 測位開始
            pushPositioningStart(null);
        } else {
            // 現在地測位
            pushGetLocation(null);
        }
    }

    /**
     * 位置測位の結果通知を受け取るリスナー
     */
    private final ILocationNotifyListener mLocationNotifyListener = new ILocationNotifyListener() {
        /**
         * 「測位結果（OKパターン）」通知（連続測位、単測位）
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedLocationOK(Intent intent) {
            //位置情報のIntentから測位情報を文字列に変換する
            String locationInfo = LocationLibIntentAnalyzer.getLocationResultOK(intent);
            if (LocationLibConst.INFORMATION_LOCATION_RESULT.equals(intent.getStringExtra(LocationLibConst.KEY_INFORMATION))) {
                //単測位
                final AlertDialog.Builder builder = new AlertDialog.Builder(MainActivity.this);
                builder.setMessage(locationInfo).setTitle(getString(R.string.dlg_single_location_title));
                builder.create().show();
            } else {
                //連続測位
                addResult(R.string.main_label_geolocation_result, locationInfo);
            }
        }

        /**
         * 「測位結果（NGパターン）」通知（連続測位、単測位）
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedLocationNG(Intent intent) {
            String message = LocationLibIntentAnalyzer.getLocationResultNG(intent);

            Toast.makeText(MainActivity.this, message, Toast.LENGTH_LONG).show();

            if (LocationLibConst.INFORMATION_MEASURE.equals(intent.getStringExtra(LocationLibConst.KEY_INFORMATION))) {
                //連続測位
                addResult(R.string.main_label_geolocation_result, message);
            }
        }

        /**
         * 「状態変更」通知
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedStateChanged(Intent intent) {
            String stateString = LocationLibIntentAnalyzer.getStateChanged(intent);
            addResult(R.string.main_label_geolocation_status, stateString);
        }

        /**
         * 「測位開始時の設定情報」通知
         *  測位開始した時に位置測位ライブラリから送られる
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedCurrentSettingInfo(Intent intent) {
            if (mStartLocationPositioningIntent == null) {
                //再起動時にライブラリから情報が送られてくるが, mStartLocationPositioningIntentがnullだったら画面表示は更新しない
                return;
            }

            //起動パラメータ表示用にIntentから文字列を作成する
            String settingString = LocationLibIntentAnalyzer.getLocationStartSetting(mStartLocationPositioningIntent, intent);
            addResult(R.string.main_label_startup_parameter, settingString);

            //状態表示用にIntentから文字列を作成する
            String stateString = LocationLibIntentAnalyzer.getLocationStartState(intent);
            addResult(R.string.main_label_geolocation_status, stateString);
        }

        /**
         * 「認証エラー」通知
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedAuthError(Intent intent) {
            //状態表示用にIntentから文字列を作成する
            String stateString = LocationLibIntentAnalyzer.getAuthErrorState(intent);
            addResult(R.string.main_label_geolocation_status, stateString);
        }

        /**
         * 「バージョン情報取得」通知
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedGetVersion(Intent intent) {
            //バージョン情報のIntentを解析する
            String string = LocationLibIntentAnalyzer.getVersionIntent(intent);
            Toast.makeText(MainActivity.this, string, Toast.LENGTH_LONG).show();
        }

        /**
         * 「設定値取得」通知
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedGetLibrarySettings(Intent intent) {
            intent.setClass(getApplication(), SettingConfirmActivity.class);
            startActivity(intent);
        }

        /**
         * 「内部エラー」通知
         *
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedInternalError(Intent intent) {
            String stateString = LocationLibIntentAnalyzer.getInternalError(intent);
            addResult(R.string.main_label_geolocation_status, stateString);
        }

        /**
         * 「オペレーション実行エラー」通知
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedOperationError(Intent intent) {
            Toast.makeText(MainActivity.this, getString(R.string.toast_operation_execution_error), Toast.LENGTH_LONG).show();
        }

        /**
         * [Debug] 指定した精度閾値により測位情報が捨てられた際の通知
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedDiscardedLocation(Intent intent) {
            //位置情報のIntentから即情報を文字列に変換する
            String locationInfoString = LocationLibIntentAnalyzer.getLocationResultOK(intent);
            addResult(R.string.discarded_area_notice_location_title, locationInfoString);
        }
    };

    /**
     * 結果リストに新規行を追加
     *
     * @param titleId タイトルのリソースID
     * @param text 内容
     */
    private void addResult(int titleId, String text){
        if (TextUtils.isEmpty(text)) {
            mResultList.add(0, String.format("[ %s : %s ]", getString(titleId), now()));
        } else {
            mResultList.add(0, String.format("[ %s : %s ]\n%s", getString(titleId), now(), text));
        }
        mResultListAdapter.notifyDataSetChanged();
    }

    /**
     * 結果リストをクリア
     */
    private void clearListView() {
        mResultList.clear();
        mResultListAdapter.notifyDataSetChanged();
    }

    /**
     * 結果リストのアダプタ設定
     */
    protected void setResultListAdapter(){
        mResultListAdapter = new ResultListAdapter();
        mResultView.setAdapter(mResultListAdapter);
    }

    /**
     * 結果リストのアダプタクラス
     */
    private class ResultListAdapter extends BaseAdapter {
        @Override
        public int getCount() {
            return mResultList.size();
        }

        @Override
        public Object getItem(int position) {
            return mResultList.get(position);
        }

        @Override
        public long getItemId(int position) {
            return position;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            View cv = convertView;
            if (cv == null ) {
                LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                cv = inflater.inflate(R.layout.row_result, mResultView, false);
            }
            String data = (String)getItem(position);
            if (!TextUtils.isEmpty(data)) {
               ((TextView) cv.findViewById(R.id.result)).setText(data);
            }
            return cv;
        }
    }

    /**
     * 現在日時を取得
     * @return String 現在日時（yyyy/MM/dd HH:mm:ss）
     */
    private String now() {
        final DateFormat df = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss", Locale.JAPAN);
        final Date date = new Date(System.currentTimeMillis());
        return df.format(date);
    }
}