package net.zenrindatacom.locationservice.sample.application;

import android.annotation.SuppressLint;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.util.Log;

import net.zenrindatacom.locationservice.sample.R;
import net.zenrindatacom.locationservice.sample.history.HistoryManager;
import net.zenrindatacom.locationservice.sample.history.HistoryManager.IHistoryNotifyListener;
import net.zenrindatacom.locationservice.sample.locationlib.LocationLibConst;
import net.zenrindatacom.locationservice.sample.preference.AppPreference;

import java.util.Date;

/**
 * アプリケーションクラス
 *
 * @author K.Hatano
 */
public class SampleApplication extends Application {

    private static final String TAG = "SampleApplication";

    @SuppressLint("StaticFieldLeak")
    private static Context mContext;

    /** 位置測位ライブラリからの通知をキャスト通知するインタフェース */
    private ILocationNotifyListener mLocationNotifyListener;

    /** 位置履歴のサーバ送信結果をキャスト通知するインタフェース */
    private IHistoryNotifyListener mHistoryNotifyListener;


    /**
     * 位置測位ライブラリからの通知をキャストした結果を受け取るリスナーを設定する
     *
     * @param listener リスナー
     */
    public void setLocationNotifyListener(ILocationNotifyListener listener) {
        mLocationNotifyListener = listener;
    }

    /**
     * 位置履歴のサーバ送信結果を受け取るリスナーを設定する
     *
     * @param listener リスナー
     */
    public void setHistoryNotifyListener(IHistoryNotifyListener listener) {
        mHistoryNotifyListener = listener;
    }
    /**
     * onCreate
     */
    @Override
    public void onCreate() {
        super.onCreate();
        mContext = getApplicationContext();
    }

    /**
     * Contextを返す
     *
     * @return Context
     */
    public static Context getAppContext() {
        return mContext;
    }

    /**
     * 通知情報をMainActivityにキャストする
     *
     * @param intent NO DESCRIPTION
     */
    public void notifyInformation(final Intent intent) {

        final String information = intent.getStringExtra(LocationLibConst.KEY_INFORMATION);
        if (information == null) {
            Log.e(TAG, "Information is null.");
            return;
        }

        //=======================================================================
        //「測位結果Intent（連続測位、単測位）and 測定結果OKパターン」の場合は位置をサーバに送信する
        //=======================================================================
        if(information.equals(LocationLibConst.INFORMATION_MEASURE) || information.equals(LocationLibConst.INFORMATION_LOCATION_RESULT)) {
            String result = intent.getStringExtra(LocationLibConst.KEY_RESULT);
            if (result != null && result.equals(LocationLibConst.RESULT_OK)) {
                AppPreference appPreference = AppPreference.getInstance();
                final boolean isServerSend = appPreference.getBoolean(mContext.getString(R.string.prf_app_key_is_send_location_to_server), false);
                if(isServerSend) {
                    //アプリ設定が「サーバーに送る」である場合
                    sendPositionToServer(intent);
                }
            }
        }

        //=======================================================================
        // リスナー登録先（MainActivity）へ通知する
        //=======================================================================
        if (mLocationNotifyListener == null) {
            //通知先が存在しない
            Log.i(TAG, "Listener is null.");
            return;
        }

        switch (information) {
            case LocationLibConst.INFORMATION_MEASURE:
            case LocationLibConst.INFORMATION_LOCATION_RESULT:
                //測位結果Intent（連続測位、単測位）
                String result = intent.getStringExtra(LocationLibConst.KEY_RESULT);
                if (result != null) {
                    if (result.equals(LocationLibConst.RESULT_OK)) {
                        //測定結果OKパターン
                        mLocationNotifyListener.onNotifiedLocationOK(intent);
                    } else if (result.equals(LocationLibConst.RESULT_NG)) {
                        //測定結果NGパターン
                        mLocationNotifyListener.onNotifiedLocationNG(intent);
                    }
                }
                break;
            case LocationLibConst.INFORMATION_STATE_CHANGED:
                //状態変更Intent
                mLocationNotifyListener.onNotifiedStateChanged(intent);
                break;
            case LocationLibConst.INFORMATION_CURRENT_SETTING:
                //測位開始時の設定情報Intent
                mLocationNotifyListener.onNotifiedCurrentSettingInfo(intent);
                break;
            case LocationLibConst.INFORMATION_AUTH_ERROR:
                //認証エラーIntent
                mLocationNotifyListener.onNotifiedAuthError(intent);
                break;
            case LocationLibConst.INFORMATION_GET_VERSION:
                //バージョン情報取得Intent
                mLocationNotifyListener.onNotifiedGetVersion(intent);
                break;
            case LocationLibConst.INFORMATION_GET_LIBRARY_SETTINGS:
                //設定値取得Intent
                mLocationNotifyListener.onNotifiedGetLibrarySettings(intent);
                break;
            case LocationLibConst.INFORMATION_ERROR:
                //内部エラーIntent
                mLocationNotifyListener.onNotifiedInternalError(intent);
                break;
            case LocationLibConst.INFORMATION_OPERATION_ERROR:
                //オペレーション実行エラーIntent
                mLocationNotifyListener.onNotifiedOperationError(intent);
                break;
            case LocationLibConst.INFORMATION_DISCARDED_MEASURE:
                //指定した精度閾値により測位情報が捨てられたIntent(for Debug)
                mLocationNotifyListener.onNotifiedDiscardedLocation(intent);
                break;
            default:
                Log.e(TAG, "Error!! (Unknown notification)");
        }
    }

    /**
     * 位置をサーバーへ送信する
     *
     * @param intent 測位結果（OKパターン）Intent
     */
    private void sendPositionToServer(Intent intent) {

        //緯度
        Double latitude = intent.getDoubleExtra(LocationLibConst.KEY_LATITUDE, Double.MIN_VALUE);
        //経度
        Double longitude = intent.getDoubleExtra(LocationLibConst.KEY_LONGITUDE, Double.MIN_VALUE);
        //測位日時
        Date date;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            date = intent.getSerializableExtra(LocationLibConst.KEY_DATE, Date.class);
        } else {
            date = (Date) intent.getSerializableExtra(LocationLibConst.KEY_DATE);
        }
        //水平精度（ライブラリ側で本値が取得できていない場合は-1.0が設定されてくる）
        Float accuracy = intent.getFloatExtra(LocationLibConst.KEY_ACCURACY, -1.0f);
        //標高（ライブラリ側で本値が取得できていない場合は-1.0が設定されてくる）
        Double altitude = intent.getDoubleExtra (LocationLibConst.KEY_ALTITUDE, -1.0d);
        //速度（ライブラリ側で本値が取得できていない場合は-1.0が設定されてくる）
        Float speed = intent.getFloatExtra(LocationLibConst.KEY_SPEED, -1.0f);
        //方位（ライブラリ側で本値が取得できていない場合は-1.0が設定されてくる）
        Float bearing = intent.getFloatExtra(LocationLibConst.KEY_BEARING, -1.0f);

        HistoryManager.Position position = new HistoryManager.Position(
                latitude,  //緯度
                longitude, //経度
                date,      //測位日時
                accuracy,  //水平精度
                altitude,  //標高
                speed,     //速度
                bearing    //方位
        );

        //サーバーへ送信
        HistoryManager.getInstance().addPosition(position, mHistoryNotifyListener);
    }
}
