package net.zenrindatacom.locationservice.sample.locationlib;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.os.Build;

import net.zenrindatacom.locationservice.sample.R;
import net.zenrindatacom.locationservice.sample.application.SampleApplication;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 位置測位ライブラリのIntentを解析するクラス
 *
 * @author K.Hatano
 */
public class LocationLibIntentAnalyzer {

    /**
     * 「測位結果（OKパターン）」Intentから結果文字列を取得する
     *
     * @param intent 位置測位ライブラリから通知された「測位結果（OKパターン）」Intent
     * @return 測位結果の文字列
     */
    @SuppressLint("SimpleDateFormat")
    public static String getLocationResultOK(Intent intent) {
        StringBuffer strBuffer = new StringBuffer();
        addObjectToBuffer(strBuffer, LocationLibConst.KEY_PROVIDER, intent.getStringExtra(LocationLibConst.KEY_PROVIDER));
        addDoubleToBuffer(strBuffer, LocationLibConst.KEY_LATITUDE, intent.getDoubleExtra(LocationLibConst.KEY_LATITUDE, Double.MIN_VALUE));
        addDoubleToBuffer(strBuffer, LocationLibConst.KEY_LONGITUDE, intent.getDoubleExtra(LocationLibConst.KEY_LONGITUDE, Double.MIN_VALUE));
        addFloatToBuffer(strBuffer, LocationLibConst.KEY_ACCURACY, intent.getFloatExtra(LocationLibConst.KEY_ACCURACY, Float.MIN_VALUE));
        addDoubleToBuffer(strBuffer, LocationLibConst.KEY_ALTITUDE, intent.getDoubleExtra(LocationLibConst.KEY_ALTITUDE, Double.MIN_VALUE));
        addFloatToBuffer(strBuffer, LocationLibConst.KEY_SPEED, intent.getFloatExtra(LocationLibConst.KEY_SPEED, Float.MIN_VALUE));
        addFloatToBuffer(strBuffer, LocationLibConst.KEY_BEARING, intent.getFloatExtra(LocationLibConst.KEY_BEARING, Float.MIN_VALUE));
        Date date;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            date = intent.getSerializableExtra(LocationLibConst.KEY_DATE, Date.class);
        } else {
            date = (Date) intent.getSerializableExtra(LocationLibConst.KEY_DATE);
        }

        if (date != null) {
            String dateString = new SimpleDateFormat("yyyyMMdd").format(date);
            addObjectToBuffer(strBuffer, "Date", dateString);
            String time = new SimpleDateFormat("HHmmss").format(date);
            addObjectToBuffer(strBuffer, "Time", time);
        }

        return trimEndLineFeed(strBuffer.toString());
    }

    /**
     * 「測位結果（NGパターン）」Intentから結果文字列を取得する
     *
     * @param intent 位置測位ライブラリから通知された「測位結果（NGパターン）」Intent
     * @return NGメッセージ
     */
    public static String getLocationResultNG(Intent intent) {
        Context ctx = SampleApplication.getAppContext();
        String cause = intent.getStringExtra(LocationLibConst.KEY_CAUSE);
        String messageFormat = ctx.getString(R.string.toast_location_result_ng);

        return String.format(messageFormat, cause);
    }

    /**
     * 「状態変更」Intentから状態文字列を取得する
     *
     * @param intent 位置測位ライブラリから通知された「状態変更」Intent
     * @return 状態の文字列
     */
    public static String getStateChanged(Intent intent) {
        Context ctx = SampleApplication.getAppContext();
        StringBuffer strBuffer = new StringBuffer();

        int stateInt = intent.getIntExtra(LocationLibConst.KEY_CHANGED_STATE, Integer.MIN_VALUE);
        int stateCause = intent.getIntExtra(LocationLibConst.KEY_STATE_CHANGE_CAUSE, Integer.MIN_VALUE);
        int stateCauseResult = intent.getIntExtra(LocationLibConst.KEY_STATE_CHANGE_CAUSE_RESULT, Integer.MIN_VALUE);
        if (stateInt != Integer.MIN_VALUE) {
            //「状態」設定
            addObjectToBuffer(strBuffer, ctx.getString(R.string.state_area_label_state), stateToString(stateInt));
        }
        if (stateCause != Integer.MIN_VALUE) {
            //「変更した原因」を元に「理由」を設定
            addObjectToBuffer(strBuffer, ctx.getString(R.string.state_area_label_reason), stateChangeCauseToString(stateCause));
            if (stateCauseResult != Integer.MIN_VALUE) {
                //「変更した原因」と「変更した理由」を元に「詳細」を設定
                addObjectToBuffer(strBuffer, ctx.getString(R.string.state_area_label_detail), stateChangeCauseResultToString(stateCause, stateCauseResult));
            }
        }

        return trimEndLineFeed(strBuffer.toString());
    }

    /**
     * Intentから測位開始時の設定情報文字列を取得する
     *
     * @param intent ライブラリへ渡す「測位開始時Intent」
     * @param settingInfo ライブラリへ渡す「測位開始時Intent」 or ライブラリから通知される「測位開始時の設定情報Intent」
     * @return 設定情報の文字列
     */
    public static String getLocationStartSetting(Intent intent, Intent settingInfo) {
        Context ctx = SampleApplication.getAppContext();
        StringBuffer strBuffer = new StringBuffer();

        //Intentの種類
        addObjectToBuffer(strBuffer, ctx.getString(R.string.launch_param_intent_operation), intent.getStringExtra(LocationLibConst.KEY_OPERATION));
        //プロバイダ種別
        String[] providers;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            providers = intent.getSerializableExtra(LocationLibConst.KEY_PROVIDER, String[].class);
        } else {
            providers = (String[]) intent.getSerializableExtra(LocationLibConst.KEY_PROVIDER);
        }
        addStringListToBuffer(strBuffer, ctx.getString(R.string.launch_param_provider), providers);

        //測位間隔
        addIntToBuffer(strBuffer, ctx.getString(R.string.launch_param_positioning_Interval), intent.getIntExtra(LocationLibConst.KEY_INTERVAL, Integer.MIN_VALUE));
        //測位精度
        addFloatToBuffer(strBuffer, ctx.getString(R.string.launch_param_positioning_accuracy), intent.getFloatExtra(LocationLibConst.KEY_ACCURACY_THRESHOLD, Float.MIN_VALUE));
        //測位結果通知アプリのパッケージ名
        addObjectToBuffer(strBuffer, ctx.getString(R.string.launch_param_notify_package), intent.getStringExtra(LocationLibConst.KEY_PACKAGE_NAME));
        //測位結果通知アプリのクラス名
        addObjectToBuffer(strBuffer, ctx.getString(R.string.launch_param_notify_class), intent.getStringExtra(LocationLibConst.KEY_CLASS_NAME));
        //デバッグモード
        addObjectToBuffer(strBuffer, ctx.getString(R.string.launch_param_debug_mode), intent.getBooleanExtra(LocationLibConst.KEY_DEBUG, false));

        //同一位置判定距離
        addIntToBuffer(strBuffer, ctx.getString(R.string.launch_param_same_position_determining_distance), settingInfo.getIntExtra(LocationLibConst.KEY_SAME_POSITION_DETERMINING_DISTANCE, Integer.MIN_VALUE));
        //同一位置判定回数
        addIntToBuffer(strBuffer, ctx.getString(R.string.launch_param_same_position_determining_count), settingInfo.getIntExtra(LocationLibConst.KEY_SAME_POSITION_DETERMINING_COUNT, Integer.MIN_VALUE));

        //APIキー
        addObjectToBuffer(strBuffer, ctx.getString(R.string.launch_param_api_key), intent.getStringExtra(LocationLibConst.KEY_API_KEY));
        //クライアントID
        addObjectToBuffer(strBuffer, ctx.getString(R.string.launch_param_client_id), intent.getStringExtra(LocationLibConst.KEY_CLIENT_ID));
        //秘密鍵
        String secretKey = intent.getStringExtra(LocationLibConst.KEY_SECRET);
        if (secretKey == null) secretKey = " ";
        addObjectToBuffer(strBuffer, ctx.getString(R.string.launch_param_secret_key), secretKey);
        //認証サーバ環境
        addIntToBuffer(strBuffer, ctx.getString(R.string.launch_param_auth_env), intent.getIntExtra(LocationLibConst.KEY_AUTH_ENV, Integer.MIN_VALUE));

        return trimEndLineFeed(strBuffer.toString());
    }

    /**
     * Intentから測位開始時の状態文字列を取得する
     *
     * @param intent 位置測位ライブラリから通知される「測位開始時の設定情報Intent」
     * @return 状態文字列
     */
    public static String getLocationStartState(Intent intent) {
        Context ctx = SampleApplication.getAppContext();
        String stateString;
        int settingErrorState = intent.getIntExtra(LocationLibConst.KEY_SETTING_ERROR, LocationLibConst.CURRENT_SETTINGS_ERROR_STATE_NO_ERROR);
        if (LocationLibConst.CURRENT_SETTINGS_ERROR_STATE_PERMISSION == settingErrorState) {
            //パーミッションチェックでエラー
            stateString = ctx.getString(R.string.state_area_setting_error_permission);
        } else {
            //エラーなし
            //　→　状態の文字列を通常測位で初期化
            stateString = stateToString(intent.getIntExtra(LocationLibConst.KEY_CURRENT_STATE, Integer.MIN_VALUE));
        }
        return stateString;
    }

    /**
     * Intentから認証エラー時の状態文字列を取得する
     *
     * @param intent 位置測位ライブラリから通知される「認証エラー時のIntent」
     * @return 状態文字列
     */
    public static String getAuthErrorState(Intent intent) {
        Context ctx = SampleApplication.getAppContext();

        String stateString;
        int errorState = intent.getIntExtra(LocationLibConst.KEY_AUTH_ERROR_STATE, -1);
        switch(errorState) {
            case LocationLibConst.AUTH_ERROR_STATE_GET_TOKEN_AUTH_FAILED:
                //トークン発行認証失敗
                stateString = ctx.getString(R.string.state_area_auth_error_state_token_issue);
                break;
            case LocationLibConst.AUTH_ERROR_STATE_AUTH_NOT_AUTHORIZED:
                //認証不許可
                stateString = ctx.getString(R.string.state_area_auth_error_state_not_authorized);
                break;
            default:
                stateString = "ErrorState unknown";
                break;
        }

        StringBuffer strBuffer = new StringBuffer();
        strBuffer.append(stateString).append("\n");
        addObjectToBuffer(strBuffer, ctx.getString(R.string.state_area_label_detail), intent.getStringExtra(LocationLibConst.KEY_AUTH_ERROR_MESSAGE));

        return trimEndLineFeed(strBuffer.toString());
    }

    /**
     * 「バージョン情報取得」Intentからバージョン情報を取得する
     *
     * @param intent 位置測位ライブラリから通知される「バージョン情報取得Intent」
     * @return バージョンの文字列
     */
    public static String getVersionIntent(Intent intent) {
        Context ctx = SampleApplication.getAppContext();
        return ctx.getString(R.string.toast_version) + " : " + intent.getStringExtra(LocationLibConst.KEY_VERSION);
    }

    /**
     * 「内部エラー」Intentからエラー情報を取得する
     *
     * @param intent 位置測位ライブラリから通知される「内部エラーIntent」
     * @return 内部エラー文字列
     */
    public static String getInternalError(Intent intent) {
        String message;
        Context ctx = SampleApplication.getAppContext();

        //「エラー内容」に応じて文字列を設定
        int errorState = intent.getIntExtra(LocationLibConst.KEY_ERROR_STATUS, 1);
        if (errorState == LocationLibConst.INTERNAL_ERROR_STATUS_NOT_START_LOCATION) {
            //予期せぬエラーによる測位停止
            message = ctx.getString(R.string.state_area_internal_error_unexpected);
        } else {
            //未知のエラー
            message = ctx.getString(R.string.state_area_internal_error_unknown);
        }

        return message;
    }

    /**
     * 状態変更Intentの「変更した状態（ChangedState）」を文字列に変換する
     *
     * @param state 状態変更Intentの「変更した状態（ChangedState）」
     * @return 「変更した状態」の文字列
     */
    private static String stateToString(int state) {
        Context ctx = SampleApplication.getAppContext();
        switch (state) {
            case LocationLibConst.STATE_CHANGED_STATE_INITIAL :
                //初期状態
                return ctx.getString(R.string.state_area_state_initial);
            case LocationLibConst.STATE_CHANGED_STATE_POSITIONING:
                //通常測位状態
                return ctx.getString(R.string.state_area_state_normal);
            case LocationLibConst.STATE_CHANGED_STATE_POSITIONING_STOPPED:
                //測位停止状態
                return ctx.getString(R.string.state_area_state_stop);
            default:
                return "ChangedState unknown";
        }
    }

    /**
     * 状態変更Intentの「変更した原因（StateChangeCause）」を文字列に変換する
     *
     * @param cause 状態変更Intentの「変更した原因（StateChangeCause）」
     * @return 「変更した原因」の文字列
     */
    private static String stateChangeCauseToString(int cause) {
        Context ctx = SampleApplication.getAppContext();
        if (cause == LocationLibConst.STATE_CHANGED_CAUSE_SAME_POSITION) {
            //同一位置関連
            return ctx.getString(R.string.state_area_cause_same_position);
        } else {
            return "StateChangeCause unknown";
        }
    }

    /**
     * 状態変更Intentの「変更した原因（StateChangeCause）」と「変更した理由（StateChangeCauseResult）」から文字列に変換する
     *
     * @param cause 状態変更Intentの「変更した原因（StateChangeCause）」
     * @param result 状態変更Intentの「変更した理由（StateChangeCauseResult）」
     * @return 「変更した原因と理由」の文字列
     */
    private static String stateChangeCauseResultToString(int cause, int result) {
        if (cause == LocationLibConst.STATE_CHANGED_CAUSE_SAME_POSITION) {
            return samePositionResultToString(result);
        } else {
            return "StateChangeCause unknown";
        }
    }

    /**
     * 状態変更Intentの「変更した原因（StateChangeCause）」が「同一位置関連」の場合に
     * 引数「変更した理由（StateChangeCauseResult）」から文字列に変換する
     *
     * @param result 状態変更Intentの「変更した理由（StateChangeCauseResult）」
     * @return 「変更した原因と理由」の文字列
     */
    private static String samePositionResultToString(int result) {
        Context ctx = SampleApplication.getAppContext();
        switch (result) {
            case LocationLibConst.STATE_CHANGED_RESULT_SAME_POSITION_IN:
                //同一位置に入った
                return ctx.getString(R.string.state_area_same_position_in);
            case LocationLibConst.STATE_CHANGED_RESULT_SAME_POSITION_OUT:
                //同一位置から出た
                return ctx.getString(R.string.state_area_same_position_out);
            default:
                return "StateChangeCauseResult unknown";
        }
    }

    private static void addObjectToBuffer(StringBuffer strBuffer, String key, Object value) {
        if (key != null && value != null) {
            strBuffer.append(key).append(" : ").append(value).append("\n");
        }
    }

    private static void addIntToBuffer(StringBuffer strBuffer, String key, Integer value) {
        if (key != null && value != null && value != Integer.MIN_VALUE) {
            strBuffer.append(key).append(" : ").append(value).append("\n");
        }
    }

    private static void addDoubleToBuffer(StringBuffer strBuffer, String key, Double value) {
        if (key != null && value != null && value != Double.MIN_VALUE) {
            strBuffer.append(key).append(" : ").append(value).append("\n");
        }
    }

    private static void addFloatToBuffer(StringBuffer strBuffer, String key, Float value) {
        if (key != null && value != null && value != Float.MIN_VALUE) {
            strBuffer.append(key).append(" : ").append(value).append("\n");
        }
    }

    private static void addStringListToBuffer(StringBuffer strBuffer, String key, String[] value) {
        if (value != null && key != null) {
            String delimiter = ",";
            strBuffer.append(key).append(" : ");
            for (String s : value) {
                strBuffer.append(delimiter).append(s);
            }
            strBuffer.deleteCharAt(strBuffer.indexOf(delimiter)).append("\n");
        }
    }

    /**
     * 文字列末尾の改行コードを除去
     * @param text  文字列
     * @return 改行コード除去後の文字列
     */
    private static String trimEndLineFeed(String text) {
        if (text != null && text.endsWith("\n")) {
            return text.substring(0,text.length() -1);
        }
        return text;
    }
}
