package net.zenrindatacom.locationservice.sample.preference;

import android.content.Context;
import android.content.SharedPreferences;

import net.zenrindatacom.locationservice.sample.R;
import net.zenrindatacom.locationservice.sample.application.SampleApplication;

/**
 * 「アプリ設定」プリファレンス
 *
 * @author N.Miyahara
 */
public class AppPreference {

    /** プリファレンスが初期化済かを表すキー */
    private static final String IS_PREFERENCE_INITIALIZED = "isInitialized";

    /** 本クラスのインスタンス */
    private static final AppPreference self = new AppPreference();

    /** 「アプリ設定」プリファレンス名（ファイル名） */
    private final String preferenceName;

    /** 「アプリ設定」プリファレンスのSharedPreferencesインスタンス */
    private final SharedPreferences sharedPreferences;

    /**
     * コンストラクタ
     */
    private AppPreference() {
        Context ctx = SampleApplication.getAppContext();
        preferenceName = ctx.getPackageName() + "_app_setting_preferences";
        sharedPreferences = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
    }

    /**
     * 本クラスのインスタンスを取得する
     * @return 本クラスのインスタンス
     */
    public static AppPreference getInstance() {
        return self;
    }

    /**
     * 「アプリ設定」プリファレンス名（ファイル名）を取得する
     * @return プリファレンス名
     */
    public String getPreferenceName() {
        return preferenceName;
    }

    /**
     * 「アプリ設定」プリファレンス初期化
     * （プリファレンスが作成されてなければ初期データを設定する）
     */
    public void init() {

        if (sharedPreferences.getBoolean(IS_PREFERENCE_INITIALIZED, false)) {
            //初期化済なら何もしない
            return;
        }

        Context ctx = SampleApplication.getAppContext();

        SharedPreferences.Editor editor = sharedPreferences.edit();

        //位置をサーバーに送信する（サーバー送信は、あくまでオプション機能の扱いなので、デフォルトOFF）
        editor.putBoolean(ctx.getString(R.string.prf_app_key_is_send_location_to_server), false);
        //位置蓄積件数
        editor.putString(ctx.getString(R.string.prf_app_key_position_accumulation_count), "1");
        //クライアントID
        editor.putString(ctx.getString(R.string.prf_app_key_client_id), "");
        //秘密鍵
        editor.putString(ctx.getString(R.string.prf_app_key_secret), "");
        //トークン発行サーバURL
        editor.putString(ctx.getString(R.string.prf_app_key_token_server_url), "");
        //APIキー
        editor.putString(ctx.getString(R.string.prf_app_key_api_key), "");
        //位置蓄積サーバー（の位置登録API）URL
        editor.putString(ctx.getString(R.string.prf_app_key_location_server_url), "");
        //端末ID
        editor.putString(ctx.getString(R.string.prf_app_key_device_id), "");
        //付加情報
        editor.putString(ctx.getString(R.string.prf_app_key_extra), "");
        //非検索付加情報
        editor.putString(ctx.getString(R.string.prf_app_key_un_searchable_extra), "");
        //検索DB可能フラグ
        editor.putBoolean(ctx.getString(R.string.prf_app_key_search_db_flag), true);

        //プリファレンスに初期データを設定済
        editor.putBoolean(IS_PREFERENCE_INITIALIZED, true);

        editor.apply();
    }

    /**
     * 「アプリ設定」プリファレンスを初期データに戻す
     */
    public void returnToDefault() {
        sharedPreferences.edit().putBoolean(IS_PREFERENCE_INITIALIZED, false).apply();
        init();
    }

    /**
     * プリファレンス値(boolean型)の取得
     * @param key キー
     * @param defaultValue デフォルト値
     * @return プリファレンス値
     */
    public boolean getBoolean(String key, boolean defaultValue) {
        return sharedPreferences.getBoolean(key, defaultValue);
    }

    /**
     * プリファレンス値(String型)の取得
     * @param key キー
     * @param defaultValue デフォルト値
     * @return プリファレンス値
     */
    public String getString(String key, String defaultValue) {
        return sharedPreferences.getString(key, defaultValue);
    }
}
