package net.zenrindatacom.locationservice.sample.preference;

import android.content.Context;
import android.content.SharedPreferences;

import net.zenrindatacom.locationservice.sample.R;
import net.zenrindatacom.locationservice.sample.application.SampleApplication;

/**
 * 「ライブラリ設定」プリファレンス
 *
 * @author N.Miyahara
 */
public class LocationLibPreference {

    /** プリファレンスが初期化済かを表すキー */
    private static final String IS_PREFERENCE_INITIALIZED = "isInitialized";

    /** 本クラスのインスタンス */
    private static final LocationLibPreference self = new LocationLibPreference();

    /** 「ライブラリ設定」プリファレンス名（ファイル名） */
    private final String preferenceName;

    /** 「ライブラリ設定」プリファレンスのSharedPreferencesインスタンス */
    private final SharedPreferences sharedPreferences;

    /**
     * コンストラクタ
     */
    private LocationLibPreference() {
        Context ctx = SampleApplication.getAppContext();
        preferenceName = ctx.getPackageName() + "_location_lib_setting_preferences";
        sharedPreferences = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
    }

    /**
     * 本クラスのインスタンスを取得する
     * @return 本クラスのインスタンス
     */
    public static LocationLibPreference getInstance() {
        return self;
    }

    /**
     * 「ライブラリ設定」プリファレンス名（ファイル名）を取得する
     * @return プリファレンス名
     */
    public String getPreferenceName() {
        return preferenceName;
    }

    /**
     * 「ライブラリ設定」プリファレンス初期化
     * （プリファレンスが作成されてなければ初期データを設定する）
     */
    public void init() {

        if (sharedPreferences.getBoolean(IS_PREFERENCE_INITIALIZED, false)) {
            //初期化済なら何もしない
            return;
        }

        Context ctx = SampleApplication.getAppContext();

        SharedPreferences.Editor editor = sharedPreferences.edit();

        //APIキー
        editor.putString(ctx.getString(R.string.prf_lib_key_api_key), "");
        //クライアントID
        editor.putString(ctx.getString(R.string.prf_lib_key_client_id), "");
        //秘密鍵
        editor.putString(ctx.getString(R.string.prf_lib_key_secret), "");
        //プロバイダー：GPS
        editor.putBoolean(ctx.getString(R.string.prf_lib_key_provider_gps), true);
        //プロバイダー：ネットワーク
        editor.putBoolean(ctx.getString(R.string.prf_lib_key_provider_network), true);
        //測位間隔
        editor.putString(ctx.getString(R.string.prf_lib_key_interval), "1000");
        //同一位置判定距離
        editor.putString(ctx.getString(R.string.prf_lib_key_same_position_determining_distance), "50");
        //同一位置判定回数
        editor.putString(ctx.getString(R.string.prf_lib_key_same_position_determining_count), "5");
        //測位精度
        editor.putString(ctx.getString(R.string.prf_lib_key_accuracy), "");
        //デバッグモード
        editor.putBoolean(ctx.getString(R.string.prf_lib_key_debug), true);
        //認証サーバ環境
        editor.putString(ctx.getString(R.string.prf_lib_key_auth_env), "0");

        //プリファレンスに初期データを設定済
        editor.putBoolean(IS_PREFERENCE_INITIALIZED, true);

        editor.apply();
    }

    /**
     * 「ライブラリ設定」プリファレンスを初期データに戻す
     */
    public void returnToDefault() {
        sharedPreferences.edit().putBoolean(IS_PREFERENCE_INITIALIZED, false).apply();
        init();
    }

    /**
     * プリファレンス値(boolean型)の取得
     * @param key キー
     * @param defaultValue デフォルト値
     * @return プリファレンス値
     */
    public boolean getBoolean(String key, boolean defaultValue) {
        return sharedPreferences.getBoolean(key, defaultValue);
    }

    /**
     * プリファレンス値(String型)の取得
     * @param key キー
     * @param defaultValue デフォルト値
     * @return プリファレンス値
     */
    public String getString(String key, String defaultValue) {
        return sharedPreferences.getString(key, defaultValue);
    }
}
